using System;
using System.Collections;

namespace dotSenku.Model
{
	/// <summary>
	/// Senku Game Board
	/// </summary>
	public class Board : MarshalByRefObject

	{		

		#region Instance Creation
		public Board()
		{			
		}

		/// <summary>
		/// creates a board with the specified width and height
		/// </summary>
		public Board(int aWidth, int anHeight)
		{	
			this.matrix = new Matrix(aWidth, anHeight);			 
		}

		#endregion		
				  


		#region getters and setters

		private Matrix _matrix;
		/// <summary>
		/// matrix of squares of the board
		/// </summary>		
		public Matrix matrix			
		{
			get{ return this._matrix;}
			set{ this._matrix = value;}
		}
		
						
		/// <summary>
		/// returns a collection containing the squares on the matrix
		/// </summary>		
		public ICollection squares
		{
			get
			{ 
				//return this.matrix select:[:each | each isNil not	]
				return this.matrix.noNilPositions();
			}
		}

		#endregion

		#region accessing
	
		/// <summary>
		/// returns the square located above the specified square
		/// </summary>		
		public Square above(Square aSquare)
		{		
			Point aPoint = this.positionOf(aSquare);
			return (Square) this.matrix.at(aPoint.x, aPoint.y + 1);
		}
		
		/// <summary>
		/// returns the square located above-left the specified square
		/// </summary>
		/// <param name="aSquare"></param>
		/// <returns></returns>		
		public Square aboveLeft(Square aSquare)
		{					  
			return this.left(this.above(aSquare));
		}
		
		/// <summary>
		/// returns the square located above-right the specified square
		/// </summary>
		/// <param name="aSquare"></param>
		/// <returns></returns>		
		public Square aboveRight(Square aSquare)					   
		{
			return this.right(this.above(aSquare));
		}
		
		/// <summary>
		/// returns the square located on the left side of the specified square
		/// </summary>
		/// <param name="aSquare"></param>
		/// <returns></returns>
		public Square left(Square aSquare)				 
		{		
			Point aPoint = this.positionOf(aSquare);
			return (Square) this.matrix.at(aPoint.x - 1, aPoint.y);
		}

		/// <summary>
		/// return a point that represents the position of the square in the board
		/// </summary>
		/// <param name="aSquare"></param>
		/// <returns></returns>		
		public Point positionOf(Square aSquare)
		{
			return this.matrix.positionOf(aSquare);
		}

		/// <summary>
		/// returns the square located on the right side of the specified square
		/// </summary>
		/// <param name="aSquare"></param>
		/// <returns></returns>		
		public Square right(Square aSquare)
		{
			Point aPoint = this.positionOf(aSquare);
			return (Square) this.matrix.at(aPoint.x+1, aPoint.y);
		}
		
		/// <summary>
		/// returns the square located under the specified square
		/// </summary>
		/// <param name="aSquare"></param>
		/// <returns></returns>
		public Square under(Square aSquare)
		{				  			
			Point aPoint = this.positionOf(aSquare);
			return (Square) this.matrix.at(aPoint.x, aPoint.y - 1);
		}

		
		/// <summary>
		/// returns the square located under-left the specified square
		/// </summary>
		/// <param name="aSquare"></param>
		/// <returns></returns>		
		public Square underLeft(Square aSquare)
		{
			return this.left(this.under(aSquare));
		}

		
		/// <summary>
		/// returns the square located under-right the specified square
		/// </summary>
		/// <param name="aSquare"></param>
		/// <returns></returns>
		public Square underRight(Square aSquare)
		{					   
			return this.right(this.under(aSquare));
		}

		#endregion

		#region validating
		
		/// <summary>
		/// indicates whether the specified square has an above square
		/// </summary>
		/// <param name="aSquare"></param>
		/// <returns></returns>		
		public bool hasAbove(Square aSquare)					 
		{		
			Point aPoint = this.positionOf(aSquare);
			if (aPoint.y != this.matrix.height - 1)
				return (this.above(aSquare) != null);
			else
				return false;
		}
		
		/// <summary>
		/// indicates whether the specified square has an above-left square
		/// </summary>
		/// <param name="aSquare"></param>
		/// <returns></returns>		
		public bool hasAboveLeft(Square aSquare)						 
		{
			return (this.hasAbove(aSquare)&& this.hasLeft(this.above(aSquare)));
		}

		/// <summary>
		/// indicates whether the specified square has an above-right square
		/// </summary>
		/// <param name="aSquare"></param>
		/// <returns></returns>		
		public bool hasAboveRight(Square aSquare)
		{				  
			return (this.hasAbove(aSquare) && this.hasRight(this.above(aSquare)));
		}

		
		/// <summary>
		/// indicates whether the specified square has a left side square
		/// </summary>
		/// <param name="aSquare"></param>
		/// <returns></returns>		
		public bool hasLeft(Square aSquare)
		{			
			Point aPoint = this.positionOf(aSquare);
			if (aPoint.x != 0)
				return (this.left(aSquare) != null);
			else
				return false;
		}

		
		/// <summary>
		/// indicates whether the specified square has a right square
		/// </summary>
		/// <param name="aSquare"></param>
		/// <returns></returns>		
		public bool hasRight(Square aSquare)
		{					 		
			Point aPoint = this.positionOf(aSquare);
			if (aPoint.x != this.matrix.width - 1)
				return (this.right(aSquare) != null);
			else
				return false;
		}

		/// <summary>
		/// indicates whether the specified square has an under square
		/// </summary>
		/// <param name="aSquare"></param>
		/// <returns></returns>		
		public bool hasUnder(Square aSquare)
		{
			Point aPoint = this.positionOf(aSquare);
			if ((aPoint.y != 0))
				return (this.under(aSquare) != null);
			else
				return false;
		}

		/// <summary>
		/// indicates whether the specified square has an under-left square
		/// </summary>
		/// <param name="aSquare"></param>
		/// <returns></returns>		
		public bool hasUnderLeft(Square aSquare)
		{
			return ( this.hasUnder(aSquare) && this.hasLeft(this.under(aSquare)));
		}


		/// <summary>
		/// indicates whether the specified square has an under-right square
		/// </summary>
		/// <param name="aSquare"></param>
		/// <returns></returns>
		public bool hasUnderRight(Square aSquare)
		{						  
			return (this.hasUnder(aSquare)&& this.hasRight(this.under(aSquare)));
		}

		#endregion
		
		#region configuring
		
		/// <summary>
		/// enables the specified area of the board
		/// </summary>
		/// <param name="aPoint"></param>
		/// <param name="aWidth"></param>
		/// <param name="anHeight"></param>
		public void enableRectangularAreaFrom(int x, int y, int aWidth, int anHeight)
		{
			for(int i = x; i < x + aWidth; i++)
				for(int j = y; j < y + anHeight; j++)
					if (this.matrix.at(i, j) == null)
						this.matrix.atPut(i, j, new Square());			
		}


		/// <summary>
		/// set pegs on the specified area of the board
		/// </summary>
		/// <param name="aPoint"></param>
		/// <param name="aWidth"></param>
		/// <param name="anHeight"></param>
		/// <param name="aClass"></param>
		public void setRectangularAreaFrom(int x, int y, int aWidth, int anHeight)
		{			
			for(int i = x; i < x + aWidth; i++)
				for(int j = y; j < y + anHeight; j++)
					if  ( ! ( (Square) this.matrix.at(i, j) ).isSet )
						( (Square) this.matrix.at(i, j) ).peg = new Peg();
		}																			
		
		#endregion

	}
}
